"""Main to use the atome library."""
import argparse
import json
import logging
import sys

from pykeyatome.client import AtomeClient


def main():
    """Define the main function."""
    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--username", required=True, help="Atome username")
    parser.add_argument("-p", "--password", required=True, help="Password")
    parser.add_argument("-l", "--atome_linky_number", help="atome_linky_number")
    parser.add_argument(
        "--debug", action="store_true", help="Print debug messages to stderr"
    )
    parser.add_argument(
        "action",
        type=str,
        default="live",
        help="Action",
        choices=["live", "consumption"],
    )
    args = parser.parse_args()
    if args.atome_linky_number:
        atome_linky_number = int(args.atome_linky_number)
    else:
        atome_linky_number = 1
    client = AtomeClient(args.username, args.password, atome_linky_number)

    if args.debug:
        # You must initialize logging, otherwise you'll not see debug output.
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.DEBUG)
        requests_log.propagate = True

    if args.action == "live":
        try:
            client.login()
            print(json.dumps(client.get_live(), indent=2))
        except BaseException as exp:
            print(exp)
            return 1
        finally:
            client.close_session()

    elif args.action == "consumption":
        try:
            client.login()
            print(json.dumps(client.get_consumption(), indent=2))

        except BaseException as exp:
            print(exp)
            return 1
        finally:
            client.close_session()
    else:
        print("Action not implemented %s", args.action)
        print(
            "Usage : __main__ -u username -p pwd [--debug] [live|consumption]"
        )


if __name__ == "__main__":
    sys.exit(main())



"""Class client for atome protocol."""
import json
import logging

import requests
import simplejson
from fake_useragent import UserAgent

# export const
DAILY_PERIOD_TYPE = "day"
WEEKLY_PERIOD_TYPE = "week"
MONTHLY_PERIOD_TYPE = "month"
YEARLY_PERIOD_TYPE = "year"


# internal const
COOKIE_NAME = "PHPSESSID"
API_BASE_URI = "https://esoftlink.esoftthings.com"
API_ENDPOINT_LOGIN = "/api/user/login.json"
API_ENDPOINT_LIVE = "/measure/live.json"
API_ENDPOINT_CONSUMPTION = "/consumption.json"
LOGIN_URL = API_BASE_URI + API_ENDPOINT_LOGIN

DEFAULT_TIMEOUT = 10
MAX_RETRIES = 3

_LOGGER = logging.getLogger(__name__)


class PyAtomeError(Exception):
    """Exception class."""

    pass


class AtomeClient(object):
    """The client class."""

    def __init__(
        self, username, password, atome_linky_number=1, session=None, timeout=None
    ):
        """Initialize the client object."""
        self.username = username
        self.password = password
        self._user_id = None
        self._user_reference = None
        self._session = session
        self._data = {}
        self._timeout = timeout
        # internal array start from 0 and not 1. Shift by 1.
        self._atome_linky_number = int(atome_linky_number) - 1

    def login(self):
        """Set http session."""
        if self._session is None:
            self._session = requests.session()
            # adding fake user-agent header
            self._session.headers.update({"User-agent": str(UserAgent().random)})
        return self._login()

    def _login(self):
        """Login to Atome's API."""
        error_flag = False
        payload = {"email": self.username, "plainPassword": self.password}

        try:
            req = self._session.post(
                LOGIN_URL,
                json=payload,
                headers={"content-type": "application/json"},
                timeout=self._timeout,
            )
        except OSError:
            _LOGGER.debug("Can not login to API")
            error_flag = True
        if error_flag:
            return None

        try:
            response_json = req.json()

            user_id = str(response_json["id"])
            user_reference = response_json["subscriptions"][self._atome_linky_number][
                "reference"
            ]

            self._user_id = user_id
            self._user_reference = user_reference
        except (
            KeyError,
            OSError,
            json.decoder.JSONDecodeError,
            simplejson.errors.JSONDecodeError,
        ) as e:
            _LOGGER.debug(
                "Impossible to decode response: \nResponse was: [%s] %s",
                str(e),
                str(req.status_code),
                str(req.text),
            )
            error_flag = True
        if error_flag:
            return None

        return response_json

    def get_user_reference(self):
        """Get user reference respect to linky number."""
        return self._user_reference

    def _get_info_from_server(self, url, max_retries=0):
        error_flag = False

        if max_retries > MAX_RETRIES:
            _LOGGER.debug("Can't gather proper data. Max retries exceeded.")
            error_flag = True
            return None

        try:
            req = self._session.get(url, timeout=self._timeout)

        except OSError as e:
            _LOGGER.debug("Could not access Atome's API: " + str(e))
            error_flag = True
        if error_flag:
            return None

        if req.status_code == 403:
            # session is wrong, need to relogin
            self.login()
            logging.info("Got error %s, relogging (max retries: %s)", str(req.status_code), str(max_retries))
            return self._get_info_from_server(url, max_retries + 1)

        if req.text == "":
            _LOGGER.debug("No data")
            error_flag = True
            return None

        try:
            json_output = req.json()
        except (
            OSError,
            json.decoder.JSONDecodeError,
            simplejson.errors.JSONDecodeError,
        ) as e:
            _LOGGER.debug(
                "Impossible to decode response: "
                + str(e)
                + "\nResponse was: "
                + str(req.text)
            )
            error_flag = True
        if error_flag:
            return None

        return json_output

    def get_live(self):
        """Get current data."""
        live_url = (
            API_BASE_URI
            + "/api/subscription/"
            + self._user_id
            + "/"
            + self._user_reference
            + API_ENDPOINT_LIVE
        )

        return self._get_info_from_server(live_url)

    def get_consumption(self):
        """Get current data."""
        consumption_url = (
            API_BASE_URI
            + "/apiV2/dataJSON/"
            + self._user_id
            + "/"
            + self._user_reference
            + "/3months"
        )

        return self._get_info_from_server(consumption_url)

    def close_session(self):
        """Close current session."""
        self._session.close()
        self._session = None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pykeyatome']

package_data = \
{'': ['*']}

install_requires = \
['fake-useragent>=0.1.11,<0.2.0',
 'requests>=2.22.0,<3.0.0',
 'simplejson>=3.16.0,<4.0.0']

setup_kwargs = {
    'name': 'pykeyatome',
    'version': '2.1.0',
    'description': 'A simple API for key atome electricity consumption',
    'long_description': '# KeyAtome PyPi\n![GitHub release](https://img.shields.io/github/release/jugla/pyKeyAtome)\n\nGet your energy consumption data from Atome, a Linky-compatible device made by Total/Direct-Energie.\nA account can have several linky. With this library, you can address them one by one\n\n### Installing\n```\npip install pykeyatome\n```\n\n## Use\nThe `__main__.py` is provided to show an example of use.\n\n4 types of function provided by client.py in `AtomeClient` class:\n- login : to be logged to the server\n- get_user_reference : to know which linky you have addressed \n- get_live : to retrieve live statistics (instant power)\n- get_consumption(period) : to retrieve the consumption since a period (day/week/month/year)\n\n## Acknowledgments\n* Thanks to k20human for the original inspiration with https://github.com/k20human/domoticz-atome\n* Thanks to reverse engineering of Atome IOS APP performed by BaQs.\n* This project is a fork of https://github.com/BaQs/pyAtome (seems to be unmaintained)\n\n### Breaking change\n**V1.2.0** Since this version PyAtomeError exception is no more used. Instead login return *False* if error , and live/consumption return *None*\n\n**V1.3.0** Login return *None* if error\n\n**V2.0.0** 1rst implementation of TotalEnergy V2 protocol : Login, Live, Day are available.\n',
    'author': 'jugla',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jugla/pyKeyAtome',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.0,<3.11',
}


setup(**setup_kwargs)
